'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');

var __assign = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __awaiter = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = undefined;
var isInvalidResult = function (result) {
    return !result.valid;
};
/**
 * Passed to validation functions to be called when value is valid
 */
var validFn = function () { return ({
    valid: true
}); };
/**
 * Passed to validation functions to be called when value is invalid
 * @param error An error message - why is the value invalid?
 */
var invalidFn = function (error) { return ({
    valid: false,
    error: error
}); };
/**
 * Passes the validation reporters to each entry in an object
 * @param validationMap Object of field validators { [ fieldName: string ]: ValidationFunction }
 */
var create = function (validationMap, formatter) {
    if (formatter === void 0) { formatter = (function (x) {
        return x;
    }); }
    return Object.entries(validationMap).reduce(function (out, _a) {
        var key = _a[0], value = _a[1];
        return (__assign({}, out, (_b = {}, _b[key] = value({ valid: validFn, invalid: invalidFn }, formatter), _b)));
        var _b;
    }, {});
};
// S value type
// T formatter input type
// U params type
// V message name
var useFormatter = function (formatter, msg, args) { return function (messageName, defaultMsg) {
    return formatter(msg && msg[messageName]
        ? typeof msg[messageName] === "function"
            ? msg[messageName](args)
            : msg[messageName]
        : defaultMsg, args);
}; };
/**
 * Validates that a value is at least {chars} long
 * @param chars Minimum number of characters
 * @param msg Error messages when invalid
 */
var atLeast = function (params, msg) { return function (_a, formatter) {
    var valid = _a.valid, invalid = _a.invalid;
    return function (value) {
        var format = useFormatter(formatter, msg, __assign({}, params, { value: value }));
        if (!value) {
            return invalid(format("undef", "Please enter a value"));
        }
        return value.length >= params.chars
            ? valid()
            : invalid(format("short", "Entry must be at least " + params.chars + " characters long"));
    };
}; };
/**
 * Validates that a value is at most {chars} long
 * @param chars Maximum number of characters
 * @param msg Error messages when invalid
 */
var atMost = function (params, msg) { return function (_a, formatter) {
    var valid = _a.valid, invalid = _a.invalid;
    return function (value) {
        if (!value) {
            return valid();
        }
        var format = useFormatter(formatter, msg, __assign({}, params, { value: value }));
        return value.length <= params.chars
            ? valid()
            : invalid(format("long", "Entry must be no more than " + params.chars + " characters long"));
    };
}; };
/**
 * Validates that a value is only numbers
 * @param msg Error messages when invalid
 */
var numeric = function (msg) { return function (_a, formatter) {
    var valid = _a.valid, invalid = _a.invalid;
    return function (value) {
        if (typeof value === "undefined") {
            return valid();
        }
        var format = useFormatter(formatter, msg, { value: value });
        return /^[0-9]*$/.test(value || "")
            ? valid()
            : invalid(format("nonNumeric", "Entered value must be a number"));
    };
}; };
/**
 * Combines validators so that all must be valid
 * @param validators Validators to combine
 * @param combiner How to combine error messages
 */
var all = function (validators, combiner) { return function (reporters, formatter) { return function (val) { return __awaiter(_this, void 0, void 0, function () {
    var results, errors;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, Promise.all(validators.map(function (validator) { return validator(reporters, formatter)(val); }))];
            case 1:
                results = _a.sent();
                if (results.every(function (r) { return r.valid; })) {
                    return [2 /*return*/, reporters.valid()];
                }
                errors = results.filter(isInvalidResult).map(function (r) { return r.error; });
                return [2 /*return*/, reporters.invalid(combiner ? combiner(errors) : errors.join(" and "))];
        }
    });
}); }; }; };
var validation = {
    atLeast: atLeast,
    atMost: atMost,
    all: all,
    create: create,
    numeric: numeric
};

var __extends = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign$1 = (undefined && undefined.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
/**
 * formActions allow the field to trigger functions within the parent
 * stateEngine allows the field to update and retrieve values from its parents state
 */
var makeField = function (formActions, stateEngine, resetState) {
    if (resetState === void 0) { resetState = true; }
    return _a = /** @class */ (function (_super) {
            __extends(Field, _super);
            /**
             * When Field is instantiated we need to set up its state
             * record in the parent Form
             */
            function Field(props) {
                var _this = _super.call(this, props) || this;
                /**
                 * Take a partial update to the fields state (stored in Form) and
                 * convert it into an update that can be applied to the Form state
                 * without losing other field's state
                 */
                _this.updateState = function (fieldState) {
                    var name = _this.props.name;
                    return stateEngine.set(function (state) {
                        return ({
                            fields: __assign$1({}, state.fields, (_a = {}, _a[name] = {
                                meta: {
                                    touched: fieldState.meta && "touched" in fieldState.meta
                                        ? fieldState.meta.touched
                                        : state.fields[name].meta.touched,
                                    active: fieldState.meta && "active" in fieldState.meta
                                        ? fieldState.meta.active
                                        : state.fields[name].meta.active,
                                    validation: fieldState.meta && fieldState.meta.validation
                                        ? fieldState.meta.validation
                                        : state.fields[name].meta.validation,
                                    isValidating: fieldState.meta &&
                                        "isValidating" in fieldState.meta
                                        ? fieldState.meta.isValidating
                                        : state.fields[name].meta.isValidating
                                },
                                value: "value" in fieldState
                                    ? fieldState.value
                                    : state.fields[name].value
                            }, _a))
                        });
                        var _a;
                    });
                };
                _this.handleFocus = function () {
                    return _this.updateState({
                        meta: {
                            touched: true,
                            active: true
                        }
                    });
                };
                _this.handleBlur = function () {
                    return _this.updateState({
                        meta: {
                            active: false
                        }
                    });
                };
                /**
                 * When the field changes we need to update the value in the parent Form
                 * and rerun validation. As validation can be async then Promise.all
                 * ensures that we immediately update the value even if validtion takes
                 * some time. Returned promise is not currently used.
                 */
                _this.handleChange = function (e) {
                    // Pulled into a variable so we don't lose this when the event is disposed
                    var value = e.currentTarget.value;
                    var _a = _this.props, onChange = _a.onChange, onBlur = _a.onBlur, onFocus = _a.onFocus;
                    onChange && onChange(e);
                    onBlur && onBlur(e);
                    onFocus && onFocus(e);
                    return Promise.all([
                        _this.updateState({
                            value: value
                        }).then(formActions.onChange),
                        _this.updateState({
                            meta: {
                                isValidating: true
                            }
                        }).then(function () {
                            return formActions
                                .validate(_this.props.name, value)
                                .then(function (validation$$1) {
                                return _this.updateState({
                                    meta: {
                                        validation: validation$$1,
                                        isValidating: false
                                    }
                                });
                            });
                        })
                    ]);
                };
                var initialValue = formActions.getInitialValue(props.name);
                // Setup initial state with validation as true
                resetState &&
                    _this.updateState({
                        meta: {
                            touched: false,
                            active: false,
                            validation: {
                                valid: true
                            },
                            isValidating: false
                        },
                        value: initialValue
                    });
                // Run validation on initialValue and update
                formActions
                    .validate(props.name, formActions.getInitialValue(props.name))
                    .then(function (validation$$1) {
                    _this.updateState({
                        meta: {
                            validation: validation$$1
                        }
                    });
                });
                return _this;
            }
            Field.prototype.render = function () {
                var _a = this.props, name = _a.name, render = _a.render, renderProps = _a.renderProps;
                var state = stateEngine.select(function (s) { return s.fields[name]; });
                /**
                 * If field does not have an initialValue, then it won't have a record
                 * added to form state until the constructor async setters have run, so
                 * we return null until the Form's record for this field is available
                 */
                if (!state) {
                    return null;
                }
                var value = state.value, _b = state.meta, touched = _b.touched, active = _b.active, validation$$1 = _b.validation, isValidating = _b.isValidating;
                var error = isInvalidResult(validation$$1)
                    ? validation$$1.error
                    : undefined;
                var valid = validation$$1.valid;
                var pristine = value === formActions.getInitialValue(name);
                return render({
                    ownProps: renderProps,
                    input: {
                        onChange: this.handleChange,
                        onFocus: this.handleFocus,
                        onBlur: this.handleBlur,
                        value: value
                    },
                    meta: {
                        valid: valid,
                        error: error,
                        pristine: pristine,
                        touched: touched,
                        active: active,
                        isValidating: isValidating
                    }
                });
            };
            return Field;
        }(React.Component)), _a.defaultProps = {
            renderProps: {}
        }, _a;
    var _a;
};

/**
 * Provides an implementation of StateEngine<FormState> using a
 * components internal state
 */
var componentStateEngine = function (componentInstance, initialState) {
    componentInstance.state = initialState;
    return {
        select: function (selector) { return selector(componentInstance.state); },
        set: function (update) {
            return new Promise(function (r) { return componentInstance.setState(update, r); });
        },
        get: function () { return componentInstance.state; }
    };
};

var __extends$1 = (undefined && undefined.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __awaiter$1 = (undefined && undefined.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator$1 = (undefined && undefined.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var Form = /** @class */ (function (_super) {
    __extends$1(Form, _super);
    /**
     * When a Form is instantiated, generate a state engine with initial empty
     * state - this will be filled in by Fields, and create the Field component
     * that will be injected.
     */
    function Form(props) {
        var _this = _super.call(this, props) || this;
        /**
         * Generates the Field that will be passed in InjectedFormProps
         */
        _this.makeField = function (resetField) {
            if (resetField === void 0) { resetField = true; }
            _this.Field = makeField({
                onChange: _this.handleFieldChange,
                validate: _this.validate,
                getInitialValue: _this.getInitialValue
            }, _this.stateEngine, resetField);
        };
        /**
         * Reset everything to initial values
         * State will be fleshed out by Field children when they are recreated
         */
        _this.reset = function () {
            _this.stateEngine
                .set({
                fields: {},
                submitted: false
            })
                .then(function () { return _this.makeField(); })
                .then(function () { return _this.forceUpdate(); });
        };
        /**
         * Allows Fields to get to their initial state
         */
        _this.getInitialValue = function (name) {
            return _this.props.initialValues[name] || "";
        };
        /**
         * Is every field passing validation
         */
        _this.allValid = function (validationResult) {
            return Object.values(validationResult).every(function (r) { return r.meta.validation.valid; });
        };
        /**
         * If there is a validator for field with name of {name}
         * then run it, otherwise return valid
         */
        _this.validate = function (name, value) { return __awaiter$1(_this, void 0, void 0, function () {
            var validator;
            return __generator$1(this, function (_a) {
                if (!this.props.validators) {
                    return [2 /*return*/, validFn()];
                }
                validator = this.props.validators[name];
                return [2 /*return*/, validator ? validator(value) : validFn()];
            });
        }); };
        /**
         * Called by Fields when their value changes
         * If a form onChange handler was passed as a prop, call it
         */
        _this.handleFieldChange = function () {
            _this.props.onChange(_this.stateEngine.select(function (s) { return s.fields; }));
        };
        /**
         * On submit call either props.onSubmit or props.onFailedSubmit
         * depending on current validation status
         */
        _this.handleSubmit = function (onSubmit, onFailedSubmit, valid, fields) { return function (e) {
            e && e.preventDefault();
            _this.stateEngine.set({ submitted: true });
            valid
                ? onSubmit(fields)
                : onFailedSubmit(fields);
        }; };
        _this.stateEngine =
            props.stateEngine ||
                componentStateEngine(_this, {
                    fields: {},
                    submitted: false
                });
        _this.makeField();
        return _this;
    }
    Form.prototype.componentWillReceiveProps = function (nextProps) {
        if (nextProps.validators !== this.props.validators) {
            this.makeField(false);
        }
        return true;
    };
    Form.prototype.render = function () {
        var _a = this.props, render = _a.render, onSubmit = _a.onSubmit, onSubmitFailed = _a.onSubmitFailed, name = _a.name, renderProps = _a.renderProps;
        var _b = this.stateEngine.get(), submitted = _b.submitted, fields = _b.fields;
        var valid = this.allValid(fields);
        /**
         * Filters out all keys from validationResult where valid is true,
         * if form is valid then we know this will be an empty object, so we can
         * just return that
         */
        var validationResult = valid
            ? {}
            : Object.entries(fields).reduce(function (out, _a) {
                var key = _a[0], value = _a[1];
                return Object.assign({}, out, isInvalidResult(value.meta.validation)
                    ? (_b = {}, _b[key] = value.meta.validation, _b) : {});
                var _b;
            }, {});
        var isValidating = Object.values(fields).some(function (field) { return field.meta.isValidating; });
        var submit = this.handleSubmit(onSubmit, onSubmitFailed, valid, fields);
        return render({
            ownProps: renderProps,
            meta: {
                valid: valid,
                submitted: submitted,
                errors: validationResult,
                isValidating: isValidating
            },
            Field: this.Field,
            values: fields,
            actions: {
                reset: this.reset,
                submit: submit
            },
            form: {
                name: name,
                onSubmit: submit
            }
        });
    };
    Form.defaultProps = {
        initialValues: {},
        onSubmit: function () { },
        onSubmitFailed: function () { },
        onChange: function () { },
        renderProps: {}
    };
    return Form;
}(React.Component));

exports.makeField = makeField;
exports.Form = Form;
exports.componentStateEngine = componentStateEngine;
exports.isInvalidResult = isInvalidResult;
exports.validFn = validFn;
exports.invalidFn = invalidFn;
exports.validation = validation;
