var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import * as React from "react";
import { isInvalidResult } from "./validators";
/**
 * formActions allow the field to trigger functions within the parent
 * stateEngine allows the field to update and retrieve values from its parents state
 */
export var makeField = function (formActions, stateEngine, resetState) {
    if (resetState === void 0) { resetState = true; }
    return _a = /** @class */ (function (_super) {
            __extends(Field, _super);
            /**
             * When Field is instantiated we need to set up its state
             * record in the parent Form
             */
            function Field(props) {
                var _this = _super.call(this, props) || this;
                /**
                 * Take a partial update to the fields state (stored in Form) and
                 * convert it into an update that can be applied to the Form state
                 * without losing other field's state
                 */
                _this.updateState = function (fieldState) {
                    var name = _this.props.name;
                    return stateEngine.set(function (state) {
                        return ({
                            fields: __assign({}, state.fields, (_a = {}, _a[name] = {
                                meta: {
                                    touched: fieldState.meta && "touched" in fieldState.meta
                                        ? fieldState.meta.touched
                                        : state.fields[name].meta.touched,
                                    active: fieldState.meta && "active" in fieldState.meta
                                        ? fieldState.meta.active
                                        : state.fields[name].meta.active,
                                    validation: fieldState.meta && fieldState.meta.validation
                                        ? fieldState.meta.validation
                                        : state.fields[name].meta.validation,
                                    isValidating: fieldState.meta &&
                                        "isValidating" in fieldState.meta
                                        ? fieldState.meta.isValidating
                                        : state.fields[name].meta.isValidating
                                },
                                value: "value" in fieldState
                                    ? fieldState.value
                                    : state.fields[name].value
                            }, _a))
                        });
                        var _a;
                    });
                };
                _this.handleFocus = function () {
                    return _this.updateState({
                        meta: {
                            touched: true,
                            active: true
                        }
                    });
                };
                _this.handleBlur = function () {
                    return _this.updateState({
                        meta: {
                            active: false
                        }
                    });
                };
                /**
                 * When the field changes we need to update the value in the parent Form
                 * and rerun validation. As validation can be async then Promise.all
                 * ensures that we immediately update the value even if validtion takes
                 * some time. Returned promise is not currently used.
                 */
                _this.handleChange = function (e) {
                    // Pulled into a variable so we don't lose this when the event is disposed
                    var value = e.currentTarget.value;
                    var _a = _this.props, onChange = _a.onChange, onBlur = _a.onBlur, onFocus = _a.onFocus;
                    onChange && onChange(e);
                    onBlur && onBlur(e);
                    onFocus && onFocus(e);
                    return Promise.all([
                        _this.updateState({
                            value: value
                        }).then(formActions.onChange),
                        _this.updateState({
                            meta: {
                                isValidating: true
                            }
                        }).then(function () {
                            return formActions
                                .validate(_this.props.name, value)
                                .then(function (validation) {
                                return _this.updateState({
                                    meta: {
                                        validation: validation,
                                        isValidating: false
                                    }
                                });
                            });
                        })
                    ]);
                };
                var initialValue = formActions.getInitialValue(props.name);
                // Setup initial state with validation as true
                resetState &&
                    _this.updateState({
                        meta: {
                            touched: false,
                            active: false,
                            validation: {
                                valid: true
                            },
                            isValidating: false
                        },
                        value: initialValue
                    });
                // Run validation on initialValue and update
                formActions
                    .validate(props.name, formActions.getInitialValue(props.name))
                    .then(function (validation) {
                    _this.updateState({
                        meta: {
                            validation: validation
                        }
                    });
                });
                return _this;
            }
            Field.prototype.render = function () {
                var _a = this.props, name = _a.name, render = _a.render, renderProps = _a.renderProps;
                var state = stateEngine.select(function (s) { return s.fields[name]; });
                /**
                 * If field does not have an initialValue, then it won't have a record
                 * added to form state until the constructor async setters have run, so
                 * we return null until the Form's record for this field is available
                 */
                if (!state) {
                    return null;
                }
                var value = state.value, _b = state.meta, touched = _b.touched, active = _b.active, validation = _b.validation, isValidating = _b.isValidating;
                var error = isInvalidResult(validation)
                    ? validation.error
                    : undefined;
                var valid = validation.valid;
                var pristine = value === formActions.getInitialValue(name);
                return render({
                    ownProps: renderProps,
                    input: {
                        onChange: this.handleChange,
                        onFocus: this.handleFocus,
                        onBlur: this.handleBlur,
                        value: value
                    },
                    meta: {
                        valid: valid,
                        error: error,
                        pristine: pristine,
                        touched: touched,
                        active: active,
                        isValidating: isValidating
                    }
                });
            };
            return Field;
        }(React.Component)),
        _a.defaultProps = {
            renderProps: {}
        },
        _a;
    var _a;
};
//# sourceMappingURL=Field.js.map