import { FieldValue } from "./index";
export declare type Reporter = (error?: string) => FieldResult;
export declare type ValidatorFn<T> = (x: T | undefined) => FieldResult | Promise<FieldResult>;
export declare type ValidationFnWithReporters<T = FieldValue | undefined, U = any, V = any> = (reporters: Reporters, formatter: Formatter<U, V>) => ValidatorFn<T>;
export declare type ParamaterizedValidationFn<T, U> = (...params: any[]) => ValidationFnWithReporters<T, U>;
export declare type MakeValidator<T, U, V = void> = (validationMap: Record<string, ValidationFnWithReporters<T, U>>, formatter?: Formatter<U, V>) => Record<string, ValidatorFn<T>>;
export interface Reporters {
    valid: () => ValidFieldResult;
    invalid: (a: string) => InvalidFieldResult;
}
export declare type FormValidator<T, U> = Record<string, ValidationFnWithReporters<T, U>>;
export declare type ValidationResult = Record<string, FieldResult>;
export declare type ValidationErrors = Record<string, InvalidFieldResult>;
export interface InvalidFieldResult {
    valid: false;
    error: string;
}
export interface ValidFieldResult {
    valid: true;
}
export declare type FieldResult = ValidFieldResult | InvalidFieldResult;
export declare const isInvalidResult: (result: FieldResult) => result is InvalidFieldResult;
/**
 * Passed to validation functions to be called when value is valid
 */
export declare const validFn: () => ValidFieldResult;
/**
 * Passed to validation functions to be called when value is invalid
 * @param error An error message - why is the value invalid?
 */
export declare const invalidFn: (error: string) => InvalidFieldResult;
export declare type Formatter<T, U> = (x: T, params?: U) => string;
export declare type MessageParams<T, U = FieldValue> = T & {
    value: U;
};
export declare type Message<T, U, V> = T | ((a: MessageParams<U, V>) => T) | undefined;
export interface NotUndefinedMessage<T = string, U = string> {
    undef?: Message<T, void, U>;
}
export interface AtLeastXCharsMessages<T = string, U = string> extends NotUndefinedMessage<T, U> {
    short?: Message<T, {
        chars: number;
    }, U>;
}
export interface AtMostXCharsMessages<T = string, U = string> {
    long: Message<T, {
        chars: number;
    }, U>;
}
export interface NumericMessages<T = string, U = string> extends NotUndefinedMessage<T, U> {
    nonNumeric?: Message<T, void, U>;
}
export declare type ValidationFnCreator<T, U, V, W> = (params: T, msg?: U) => ValidationFnWithReporters<V, W>;
export declare const validation: {
    atLeast: <T>(params: {
        chars: number;
    }, msg?: AtLeastXCharsMessages<T, string> | undefined) => ({valid, invalid}: Reporters, formatter: Formatter<T, MessageParams<{
        chars: number;
    }, FieldValue>>) => (value: string) => FieldResult;
    atMost: <T>(params: {
        chars: number;
    }, msg: AtMostXCharsMessages<T, string>) => ({valid, invalid}: Reporters, formatter: Formatter<T, MessageParams<{
        chars: number;
    }, FieldValue>>) => (value: string) => FieldResult;
    all: <T, U>(validators: ValidationFnWithReporters<T, U, any>[], combiner?: ((errors: string[]) => string) | undefined) => (reporters: Reporters, formatter: Formatter<U, MessageParams<FieldValue, any>>) => (val: T) => Promise<FieldResult>;
    create: <T = FieldValue, U = string>(validationMap: Record<string, ValidationFnWithReporters<T, U, any>>, formatter?: Formatter<U, MessageParams<FieldValue, any>>) => {};
    numeric: <T>(msg?: NumericMessages<T, string> | undefined) => ({valid, invalid}: Reporters, formatter: Formatter<T, {
        value: FieldValue;
    }>) => (value: string) => FieldResult;
};
