var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
export var isInvalidResult = function (result) {
    return !result.valid;
};
/**
 * Passed to validation functions to be called when value is valid
 */
export var validFn = function () { return ({
    valid: true
}); };
/**
 * Passed to validation functions to be called when value is invalid
 * @param error An error message - why is the value invalid?
 */
export var invalidFn = function (error) { return ({
    valid: false,
    error: error
}); };
/**
 * Passes the validation reporters to each entry in an object
 * @param validationMap Object of field validators { [ fieldName: string ]: ValidationFunction }
 */
var create = function (validationMap, formatter) {
    if (formatter === void 0) { formatter = (function (x) {
        return x;
    }); }
    return Object.entries(validationMap).reduce(function (out, _a) {
        var key = _a[0], value = _a[1];
        return (__assign({}, out, (_b = {}, _b[key] = value({ valid: validFn, invalid: invalidFn }, formatter), _b)));
        var _b;
    }, {});
};
// S value type
// T formatter input type
// U params type
// V message name
var useFormatter = function (formatter, msg, args) { return function (messageName, defaultMsg) {
    return formatter(msg && msg[messageName]
        ? typeof msg[messageName] === "function"
            ? msg[messageName](args)
            : msg[messageName]
        : defaultMsg, args);
}; };
/**
 * Validates that a value is at least {chars} long
 * @param chars Minimum number of characters
 * @param msg Error messages when invalid
 */
var atLeast = function (params, msg) { return function (_a, formatter) {
    var valid = _a.valid, invalid = _a.invalid;
    return function (value) {
        var format = useFormatter(formatter, msg, __assign({}, params, { value: value }));
        if (!value) {
            return invalid(format("undef", "Please enter a value"));
        }
        return value.length >= params.chars
            ? valid()
            : invalid(format("short", "Entry must be at least " + params.chars + " characters long"));
    };
}; };
/**
 * Validates that a value is at most {chars} long
 * @param chars Maximum number of characters
 * @param msg Error messages when invalid
 */
var atMost = function (params, msg) { return function (_a, formatter) {
    var valid = _a.valid, invalid = _a.invalid;
    return function (value) {
        if (!value) {
            return valid();
        }
        var format = useFormatter(formatter, msg, __assign({}, params, { value: value }));
        return value.length <= params.chars
            ? valid()
            : invalid(format("long", "Entry must be no more than " + params.chars + " characters long"));
    };
}; };
/**
 * Validates that a value is only numbers
 * @param msg Error messages when invalid
 */
var numeric = function (msg) { return function (_a, formatter) {
    var valid = _a.valid, invalid = _a.invalid;
    return function (value) {
        if (typeof value === "undefined") {
            return valid();
        }
        var format = useFormatter(formatter, msg, { value: value });
        return /^[0-9]*$/.test(value || "")
            ? valid()
            : invalid(format("nonNumeric", "Entered value must be a number"));
    };
}; };
/**
 * Combines validators so that all must be valid
 * @param validators Validators to combine
 * @param combiner How to combine error messages
 */
var all = function (validators, combiner) { return function (reporters, formatter) { return function (val) { return __awaiter(_this, void 0, void 0, function () {
    var results, errors;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0: return [4 /*yield*/, Promise.all(validators.map(function (validator) { return validator(reporters, formatter)(val); }))];
            case 1:
                results = _a.sent();
                if (results.every(function (r) { return r.valid; })) {
                    return [2 /*return*/, reporters.valid()];
                }
                errors = results.filter(isInvalidResult).map(function (r) { return r.error; });
                return [2 /*return*/, reporters.invalid(combiner ? combiner(errors) : errors.join(" and "))];
        }
    });
}); }; }; };
export var validation = {
    atLeast: atLeast,
    atMost: atMost,
    all: all,
    create: create,
    numeric: numeric
};
//# sourceMappingURL=validators.js.map